//
//  XSPEC12  November 2003
//
//

#include "xsTypes.h"
#include <XSModel/GlobalContainer/ModelContainer.h>
#include <XSUtil/FunctionUtils/funcType.h>
#include <cmath>
#include <iomanip>

#include "Projection.h"


extern "C" void project (const EnergyPointer& energyArray,
                                  const std::vector<Real>& parameterValues,
                                  GroupFluxContainer& flux,
                                  GroupFluxContainer& fluxError,
                                  MixBase** mixGenerator,
                                  const string& modName)
{
        using namespace XSContainer;
        static const string PROJECT("Projection");
        // mixBase pointers are owned by ModelContainer which is a singleton.
        ModelContainer* m(ModelContainer::Instance());

        // on first call, set the pointer addressed by mixGenerator to 
        // be returned to the calling routine.
        try
        {
           if (!*mixGenerator)
           {

                 if (!m->mixingComponents(PROJECT))
                 {
                           m->mixingComponents(PROJECT,new Projection(PROJECT));   
                 }
                 *mixGenerator = m->mixingComponents(PROJECT);
                 (*mixGenerator)->initialize(parameterValues);
           }

           // only call this if mixGenerator is 'new'.

           (*mixGenerator)->perform(energyArray,parameterValues,flux,fluxError);
        }
        catch (...)
        {
           // If it throws out of here, MixComponent::perform may never
           // get the chance to set its mixGenerator pointer.  Therefore,
           // can't rely on it to initiate destruction of the faulty
           // MixBase object.  So, do it now.
           m->removeMixingComponent(PROJECT);
           *mixGenerator = 0;
           throw;
        }
}
